function FindNearestAHuman(Pos,curdist)
	local ahuman;
	for actor in MovableMan.Actors do
		if (actor.ClassName == "AHuman") then
			local Dist=SceneMan:ShortestDistance(actor.Pos,Pos,true);
			if Dist.Magnitude < curdist then
				curdist = Dist.Magnitude;
				ahuman = actor;
			end
		end
	end
	return ahuman;
end

function ConvertAngleToUnitVector(Angle)
	return Vector(math.cos(Angle),-math.sin(Angle));
end

function ReturnPerpendicularVector(OriginalVector)
	local Result = Vector(OriginalVector.Y,-OriginalVector.X);
	return Result;
end

function ReturnRandomPositiveNegative(Number)
	if Number == nil then
		Number = 1;
	end
	Number = Number-math.random()*Number*2;
	return Number;
end

function DrawLine(StartPosition, EndPosition, DrawLengthMultiplier, DrawingQuality, MOPixelToDrawWith)	
	local VectorToDestination = SceneMan:ShortestDistance(StartPosition,EndPosition,true) * DrawLengthMultiplier;
	local UnitVector = VectorToDestination / VectorToDestination.Magnitude;
	local ParticleCount = VectorToDestination.Magnitude * DrawingQuality;
	local DistanceMultiplier = (1 / DrawingQuality);
	for ParticleNumber = 0, ParticleCount, 1 do
		local Particle = CreateMOPixel(MOPixelToDrawWith);
		Particle.Pos = StartPosition+UnitVector * DistanceMultiplier * ParticleNumber;
		MovableMan:AddParticle(Particle);
	end
end

function DrawChaoticLine(StartPosition, EndPosition, MaximumDeviationFromTheAxis, DrawLengthMultiplier, DrawingQuality, MOPixelToDrawWith)	
	local VectorToDestination = SceneMan:ShortestDistance(StartPosition,EndPosition,true) * DrawLengthMultiplier;
	local UnitVector = VectorToDestination / VectorToDestination.Magnitude;
	local OffsetVector = ReturnPerpendicularVector(UnitVector);
	local ParticleCount = VectorToDestination.Magnitude * DrawingQuality;
	local DistanceMultiplier = (1 / DrawingQuality);
	for ParticleNumber = 0, ParticleCount, 1 do
		local Particle = CreateMOPixel(MOPixelToDrawWith);
		local Offset = OffsetVector * ReturnRandomPositiveNegative(MaximumDeviationFromTheAxis);
		Particle.Pos = StartPosition + UnitVector * ParticleNumber * DistanceMultiplier + Offset;
		MovableMan:AddParticle(Particle);
	end
end

function BasicFractureVector(StartPosition,EndPosition,FragmentCount,Deviation)
	local Path = {};
	local Dest = SceneMan:ShortestDistance(StartPosition,EndPosition,true);
	local UnitVector = Dest/Dest.Magnitude;
	local Step = Dest / FragmentCount;
	local OffsetVector = ReturnPerpendicularVector(UnitVector);
	local Position = StartPosition;
	
	Path[0] = StartPosition;
	Path[FragmentCount] = EndPosition;
	for FragmentNumber = 1, FragmentCount-1, 1 do
		local Offset = OffsetVector * ReturnRandomPositiveNegative(Deviation);
		Path[FragmentNumber] = Position + Step + Offset;
		Position = Position + Step;
	end
	return Path;
end

function FractureVectorConsideringTerrainAndMOs(StartPosition,EndPosition,FragmentCount,Deviation,MOIDToIgnore)
	local Path = {};
	local Dest = SceneMan:ShortestDistance(StartPosition,EndPosition,true);
	local UnitVector = Dest/Dest.Magnitude;
	local Step = Dest / FragmentCount;
	local OffsetVector = ReturnPerpendicularVector(UnitVector);
	local Position = StartPosition;
	local MOContact = 255;
	local TerrainContact = nil;
	
	Path[0] = StartPosition;
	for FragmentNumber = 1, FragmentCount, 1 do
		local Offset = OffsetVector * ReturnRandomPositiveNegative(Deviation);
		local ModifiedStep = Step + Offset;
		local HitTerrain = SceneMan:CastNotMaterialRay(Position, ModifiedStep, 0, 5,false);
		
		if HitTerrain<0 then
			MOContact = SceneMan:CastMORay(Position, ModifiedStep, MOIDToIgnore, -1, false, 0);
			if MOContact ~= 255 and MovableMan:GetMOFromID(MOContact).GetsHitByMOs then
				Path[FragmentNumber]=MovableMan:GetMOFromID(MOContact).Pos;
				break
			end
			Path[FragmentNumber] = Position + ModifiedStep;
			Position = Position + Step;
		else
			ModifiedStep=(ModifiedStep/ModifiedStep.Magnitude)*HitTerrain;
			MOContact = SceneMan:CastMORay(Position, ModifiedStep, 255, -1, false, 0);
			if MOContact ~= 255 and MovableMan:GetMOFromID(MOContact).GetsHitByMOs then
				Path[FragmentNumber]=MovableMan:GetMOFromID(MOContact).Pos;
			else
				Path[FragmentNumber]=Position+ModifiedStep;
				TerrainContact = Path[FragmentNumber];
			end
			break
		end
	end
	return Path,MOContact,TerrainContact;
end

function DoNothing()
end

function Explosion(Pos)
	local expl = CreateMOSRotating("LDischarge");
	expl.Pos=Pos;
	MovableMan:AddParticle(expl);
end

function LastPositionExplosion(Path)
	Explosion(Path[#Path]);
end

function ExplosionOnRootMO(Path,HitMO)
	if HitMO ~= 255 and MovableMan:GetMOFromID(HitMO).GetsHitByMOs then
		local RootID = MovableMan:GetRootMOID(HitMO);
		local RootMO = MovableMan:GetMOFromID(RootID)
		DrawChaoticLine(Path[#Path],RootMO.Pos,0.5,1,1,"LboltArcOnlyBlueLightTiny");
		Explosion(RootMO.Pos);
	end
end

function CastBasicLightningArc(StartPosition,DirectionVector,ArcLength,MinimumDeviationFromTheAxis,MaximumDeviationFromTheAxis,MinimumNumberOfArcFractures,MaximumNumberOfArcFractures,MOPixelToDrawWith,DrawJitter,MinimumDrawLengthMultiplier,MaximumDrawLengthMultiplier,DrawingQuality,MOIDToIgnore,FunctionToCallOnHitMO,FunctionToCallOnHitTerrain,FunctionToCallOnHitNothing)
	local VectorToDestination = DirectionVector/DirectionVector.Magnitude*ArcLength;
	local Path;
	local HitMO = 255;
	local HitTerrain = nil;
	local NumberOfArcFractures = math.min(MaximumNumberOfArcFractures,MinimumNumberOfArcFractures)+math.ceil(math.random()*math.abs(MaximumNumberOfArcFractures-MinimumNumberOfArcFractures));
	local DeviationFromTheAxis = math.min(MaximumDeviationFromTheAxis,MinimumDeviationFromTheAxis)+math.ceil(math.random()*math.abs(MaximumDeviationFromTheAxis-MinimumDeviationFromTheAxis));

	Path,HitMO,HitTerrain = FractureVectorConsideringTerrainAndMOs(StartPosition,StartPosition+VectorToDestination,NumberOfArcFractures,DeviationFromTheAxis,MOIDToIgnore);	
	local NumberOfSteps = #Path - 1;
	for Step=0,NumberOfSteps,1 do	
		local DrawLengthMultiplier = math.min(MaximumDrawLengthMultiplier,MinimumDrawLengthMultiplier)+(math.random()*math.abs(MaximumDrawLengthMultiplier-MinimumDrawLengthMultiplier));
		if Step == NumberOfSteps then
			DrawLengthMultiplier = 1;
		end
		if DrawJitter == 0 then
			DrawLine(Path[Step],Path[Step + 1],DrawLengthMultiplier,DrawingQuality,MOPixelToDrawWith);
		else
			DrawChaoticLine(Path[Step],Path[Step + 1],DrawJitter,DrawLengthMultiplier,DrawingQuality,MOPixelToDrawWith);
		end
	end
	
	if HitMO ~= 255 then
		pcall(FunctionToCallOnHitMO,Path,HitMO,HitTerrain);
	else
		if (HitTerrain ~= nil) then
			pcall(FunctionToCallOnHitTerrain,Path,HitMO,HitTerrain);
		else
			pcall(FunctionToCallOnHitNothing,Path,HitMO,HitTerrain);
		end
	end

end

function Create(self)
	local Actor = FindNearestAHuman(self.Pos,50);
	self.Pos=self.Pos+Actor.Vel;
	CastBasicLightningArc(self.Pos,ConvertAngleToUnitVector(Actor:GetAimAngle(true)),350,3,10,17,34,"LboltArcOnlyBlueLightTiny",0.5,1,1.5,1,Actor.ID,ExplosionOnRootMO,LastPositionExplosion,DoNothing);
	self.ToDelete=true;
end

